# coding: utf-8

from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _
from mptt.models import MPTTModel, TreeForeignKey

from jmb.core.db.models import DateModel, UserModel, StatusModel


class NavItem(MPTTModel, DateModel, UserModel):
    VISIBLE = settings.NAVBAR_NAVITEM_STATUS_CODES[0]  # value 1
    INVISIBLE = settings.NAVBAR_NAVITEM_STATUS_CODES[1]  # value 0
    LOGGED_VISIBLE = settings.NAVBAR_NAVITEM_STATUS_CODES[2]  # value 2

    status = models.IntegerField(
        choices=settings.NAVBAR_NAVITEM_STATUS_CODES,
        default=1,
        verbose_name=_('Status'),
        null=True,
        help_text=_("""
        If parent is hidden, this object will set to hidden to.
        If the parent is visible only to logged in users, this object cannot be visible to all.
        If the status changes, the status of the descendants could also be changed.
        """)

    )
    is_hightlighted = models.BooleanField(
        verbose_name=_('is hightlighted'),
        default=False,
    )
    is_header = models.BooleanField(
        verbose_name=_('is a dropdown header'),
        default=False,
    )
    label = models.CharField(
        max_length=255,
        verbose_name=_('label'),
    )
    link = models.CharField(
        verbose_name=_('url relativo'),
        max_length=255,
        blank=True,
        null=True,
    )
    target = models.CharField(
        choices=settings.NAVBAR_NAVITEM_TARGET_CHOICES,
        verbose_name=_('target'),
        max_length=2,
        default='s',
    )
    html_rendered = models.TextField(
        verbose_name=_('html rendered'),
        blank=True,
        null=True,
    )

    parent = TreeForeignKey(
        'self',
        null=True,
        blank=True,
        related_name='children',
        verbose_name=_('parent'),
    )

    class Meta(object):
        verbose_name = _('Nav Item')
        verbose_name_plural = _('Nav Items')

    def __unicode__(self):
        return '%s: %s' % (self.label, self.link)

    def save(self, user=None, forced=False, *args, **kwargs):

        if not forced:
            # THIS LOGIC IS ALSO ENFORCED AT FORM LEVEL
            # if the parent is hidden, the child must be hidden too
            if self.parent:
                if self.parent.status == 0:
                    self.status = 0
                # if the parent is visible only to authenticated users, the child cannot be visible to all
                elif self.parent.status == 2:
                    if self.status == 1:
                        self.status = 2
                if not self.status == 1:
                    # if self status get changed to non 'visible', then also the child could have to be changed
                    descendants = self.get_descendants()
                    for descendant in descendants:
                        if self.status == 0:
                            descendant.status = 0
                        if self.status == 2 and descendant.status == 1:
                            descendant.status = 2
                        descendant.save(forced=True)
        super(NavItem, self).save(user, *args, **kwargs)

    def get_target_value(self):
        return settings.NAVBAR_NAVITEM_TARGET_VALUES.get(self.target, self.target)

if settings.CMS_ENABLED:
    import jmb.cms.navbar.cms.models